/*
 *	Ohio Trollius
 *	Copyright 1996 The Ohio State University
 *	NJN
 *
 *	$Id: rpisys.h,v 6.1 96/11/22 14:13:19 nevin Rel $
 *
 *	Function:	- RPI dependent structures
 */

#ifndef _RPISYS_H
#define _RPISYS_H

#include <app_mgmt.h>
#include <mpi.h>
#include <mpisys.h>
#include <rpi.lamd.h>

/*
 * Include RPI header file according to the selected interface.
 */
#if defined(RPI_SHM)

#if defined(SHM_SYSV)
#include <shm.sysv.h>
#elif defined(SHM_SGI)
#include <shm.sgi.h>
#elif defined(SHM_HPUX)
#include <shm.hpux.h>
#elif defined(SHM_SOL)
#include <shm.sol.h>
#else
Error: shared memory type undefined.
#endif

#elif defined(RPI_TCP)

#include <rpi.tcp.h>

#else

Error: RPI type undefined.

#endif

typedef struct c2c_req	c2creq_t;
typedef struct c2c_proc c2cproc_t;

union rpi_proc {
	struct c2c_proc	c2c;			/* client-to-client data */
};

struct _proc {
	struct _gps	p_gps;			/* process GPS */
	int		p_ger_nsnd;		/* #msgs sent there */
	int		p_mode;			/* mode flags */
#define LAM_PFLAG	0x001			/* used for marking */
#define LAM_PDEAD	0x002			/* node died */
#define LAM_PRPIINIT	0x004			/* RPI initialized */
#define LAM_PCLIENT	0x008			/* in client world */
	int		p_refcount;		/* reference count */
	union rpi_proc	p_rpi;			/* RPI specific stuff */
};

union rpi_req {
	struct lamd_req	lamd;			/* LAM daemon mode req. data */
	struct c2c_req	c2c;			/* client-to-client req. data */
};

struct _req {
	int		rq_type;
#define LAM_RQISEND	0
#define LAM_RQIBSEND	1
#define LAM_RQISSEND	2
#define LAM_RQIRSEND	3
#define LAM_RQIRECV	4
#define LAM_RQIPROBE	5
#define LAM_RQIFAKE	6
	int		rq_state;
#define LAM_RQSINIT	0			/* request initialized */
#define LAM_RQSSTART	1			/* active, nothing done yet */
#define LAM_RQSACTIVE	4			/* active, undone request */
#define LAM_RQSDONE	3			/* request done */
	int		rq_marks;		/* persistent flags */
#define LAM_RQFPERSIST	0x0001			/* persistent request */
#define LAM_RQFDYNBUF	0x0004			/* dynamic buffer */
#define LAM_RQFDYNREQ	0x0008			/* dynamic request */
#define LAM_RQFSOURCE	0x0200			/* source request */
#define LAM_RQFDEST	0x0400			/* destination request */
#define LAM_RQFBLKTYPE	0x0800			/* blocking request type */
	int		rq_flags;		/* active req flags */
#define LAM_RQFCANCEL	0x0002			/* cancelled request */
#define LAM_RQFBLOCK	0x0010			/* blocking request */
#define LAM_RQFTRUNC	0x0040			/* truncated message */
#define LAM_RQFORPHAN	0x0100			/* destroy when done */
#define LAM_RQFCHAR	0x0100000		/* == DRAWMSG */
#define LAM_RQFINT	0x0200000		/* == DINT4MSG */
#define LAM_RQFFLOAT	0x0400000		/* == DFLT4MSG */
#define LAM_RQFDOUBLE	0x0800000		/* == DFLT8MSG */
#define LAM_RQFSHORT	0x01000000		/* == DINT2MSG */
	char		*rq_packbuf;		/* pack buffer */
	int		rq_packsize;		/* pack buffer size */
	int		rq_count;		/* MPI request parameter */
	void		*rq_buf;		/* MPI request parameter */
	MPI_Datatype	rq_dtype;		/* MPI request parameter */
	int		rq_rank;		/* MPI request parameter */
	int		rq_tag;			/* MPI request parameter */
	MPI_Comm	rq_comm;		/* MPI request parameter */
	int		rq_cid;			/* context ID to use */
	int		rq_func;		/* MPI func. which made req. */
	int		rq_seq;			/* seq# of associated msg */
	MPI_Status	rq_status;		/* status info storage */
	struct _bsndhdr	*rq_bsend;		/* ptr bsend header */
	struct _proc	*rq_proc;		/* ptr peer process */
	struct _req	*rq_next;		/* ptr next request */
	union rpi_req	rq_rpi;			/* RPI specific stuff */
};

#endif	/* _RPISYS_H */
